@@ -1,52 +1,68 @@
+# Copyright 2021 cstsunfu. All rights reserved.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import hjson
 import pickle as pkl
 import os
 import json
-from typing import Union, Dict, Any
+import pandas as pd
+from typing import Union, Dict, Any, List
 from dlk.utils.parser import BaseConfigParser, PostProcessorConfigParser
 from dlk.data.postprocessors import postprocessor_register, postprocessor_config_register, IPostProcessor, IPostProcessorConfig
-from dlk.utils.logger import logger
+from dlk.utils.logger import Logger
 import torch
 from dlk.utils.vocab import Vocabulary
 import torchmetrics
 
-logger = logger.get_logger()
+logger = Logger.get_logger()
 
 
 @postprocessor_config_register('txt_cls')
 class TxtClsPostProcessorConfig(IPostProcessorConfig):
-    """docstring for TxtClsPostProcessorConfig
-    config e.g.
-    {
-        "_name": "txt_cls",
-        "config": {
-            "meta": "*@*",
-            "meta_data": {
-                "label_vocab": 'label_vocab',
-            },
-            "input_map": {
-                "logits": "logits",
-                "label_ids": "label_ids"
-                "_index": "_index",
-            },
-            "origin_input_map": {
-                "sentence": "sentence",
-                "sentence_a": "sentence_a", // for pair
-                "sentence_b": "sentence_b",
-                "uuid": "uuid"
-            },
-            "save_root_path": ".",  //save data root dir
-            "top_k": 1, //the result return top k result
-            "focus": [], //always return the list label values which index in 'focus' list, if the focus[0] == [1], then the predict value always return the logits[1], and the label always be label_vocab[1]
-            "data_type": "single", //single or pair
-            "save_path": {
-                "valid": "valid",  // relative dir for valid stage
-                "test": "test",    // relative dir for test stage
-            },
-            "start_save_step": 0,  // -1 means the last
-            "start_save_epoch": -1,
-        }
-    }
+    """Config for TxtClsPostProcessor
+
+    Config Example:
+        >>> {
+        >>>     "_name": "txt_cls",
+        >>>     "config": {
+        >>>         "meta": "*@*",
+        >>>         "meta_data": {
+        >>>             "label_vocab": 'label_vocab',
+        >>>         },
+        >>>         "input_map": {
+        >>>             "logits": "logits",
+        >>>             "label_ids": "label_ids"
+        >>>             "_index": "_index",
+        >>>         },
+        >>>         "origin_input_map": {
+        >>>             "sentence": "sentence",
+        >>>             "sentence_a": "sentence_a", // for pair
+        >>>             "sentence_b": "sentence_b",
+        >>>             "uuid": "uuid"
+        >>>         },
+        >>>         "save_root_path": ".",  //save data root dir
+        >>>         "top_k": 1, //the result return top k result
+        >>>         "focus": [], //always return the list label values which index in 'focus' list, if the focus[0] == 'pos', then the predict value always return the logits[vocab.get_index('pos')], and the label always be 'pos'
+        >>>         "data_type": "single", //single or pair
+        >>>         "save_path": {
+        >>>             "valid": "valid",  // relative dir for valid stage
+        >>>             "test": "test",    // relative dir for test stage
+        >>>         },
+        >>>         "start_save_step": 0,  // -1 means the last
+        >>>         "start_save_epoch": -1,
+        >>>     }
+        >>> }
     """
 
     def __init__(self, config: Dict):
@@ -99,24 +115,35 @@ class TxtClsPostProcessorConfig(IPostProcessorConfig):
 
 @postprocessor_register('txt_cls')
 class TxtClsPostProcessor(IPostProcessor):
-    """docstring for TxtClsPostProcessor"""
+    """postprocess for text classfication"""
     def __init__(self, config: TxtClsPostProcessorConfig):
         super(TxtClsPostProcessor, self).__init__()
         self.config = config
         self.label_vocab = self.config.label_vocab
         self.acc_calc = torchmetrics.Accuracy()
 
-    def do_predict(self, stage, list_batch_outputs, origin_data, rt_config):
-        """TODO: Docstring for do_predict.
-        :stage: TODO
-        :list_batch_outputs: TODO
-        :origin_data: TODO
-        :rt_config: TODO
-        :returns: TODO
+    def do_predict(self, stage: str, list_batch_outputs: List[Dict], origin_data: pd.DataFrame, rt_config: Dict)->List:
+        """Process the model predict to human readable format
+
+        Args:
+            stage: train/test/etc.
+            list_batch_outputs: a list of outputs
+            origin_data: the origin pd.DataFrame data, there are some data not be able to convert to tensor
+            rt_config: current status
+                >>> {
+                >>>     "current_step": self.global_step,
+                >>>     "current_epoch": self.current_epoch,
+                >>>     "total_steps": self.num_training_steps,
+                >>>     "total_epochs": self.num_training_epochs
+                >>> }
+
+        Returns: 
+            all predicts
+
         """
         results = []
         for outputs in list_batch_outputs:
-            logits = outputs[self.config.logits]
+            logits = outputs[self.config.logits].detach()
             assert len(logits.shape) == 2
             # predict_indexes = list(torch.argmax(logits, 1))
             indexes = list(outputs[self.config._index])
@@ -140,7 +167,8 @@ class TxtClsPostProcessor(IPostProcessor):
                 uuid = one_origin[self.config.uuid]
                 if self.config.focus:
                     label_values, label_indeies = [], []
-                    for label_index in self.config.focus:
+                    for label_name in self.config.focus:
+                        label_index = self.label_vocab.get_index(label_name)
                         label_values.append(one_logits[label_index])
                         label_indeies.append(label_index)
                 else:
@@ -159,25 +187,53 @@ class TxtClsPostProcessor(IPostProcessor):
                 results.append(one_ins)
         return results
 
-    def do_calc_metrics(self, predicts, stage, list_batch_outputs, origin_data, rt_config):
-        """TODO: Docstring for do_calc_metrics.
-        :returns: TODO
+    def do_calc_metrics(self, predicts: List, stage: str, list_batch_outputs: List[Dict], origin_data: pd.DataFrame, rt_config: Dict)->Dict:
+        """calc the scores use the predicts or list_batch_outputs
+
+        Args:
+            predicts: list of predicts
+            stage: train/test/etc.
+            list_batch_outputs: a list of outputs
+            origin_data: the origin pd.DataFrame data, there are some data not be able to convert to tensor
+            rt_config: current status
+                >>> {
+                >>>     "current_step": self.global_step,
+                >>>     "current_epoch": self.current_epoch,
+                >>>     "total_steps": self.num_training_steps,
+                >>>     "total_epochs": self.num_training_epochs
+                >>> }
+
+        Returns: 
+            the named scores, acc
 
         """
         for outputs in list_batch_outputs:
             logits = outputs[self.config.logits]
-            label_ids = outputs[self.config.label_ids]
+            label_ids = outputs[self.config.label_ids].squeeze(-1)
             self.acc_calc.update(logits, label_ids)
         real_name = self.loss_name_map(stage)
         return {f'{real_name}_acc': self.acc_calc.compute()}
 
-    def do_save(self, predicts, stage, list_batch_outputs, origin_data, rt_config={}, save_condition=False):
-        """TODO: Docstring for do_save.
+    def do_save(self, predicts: List, stage: str, list_batch_outputs: List[Dict], origin_data: pd.DataFrame, rt_config: Dict, save_condition: bool=False):
+        """save the predict when save_condition==True
+
+        Args:
+            predicts: list of predicts
+            stage: train/test/etc.
+            list_batch_outputs: a list of outputs
+            origin_data: the origin pd.DataFrame data, there are some data not be able to convert to tensor
+            rt_config: current status
+                >>> {
+                >>>     "current_step": self.global_step,
+                >>>     "current_epoch": self.current_epoch,
+                >>>     "total_steps": self.num_training_steps,
+                >>>     "total_epochs": self.num_training_epochs
+                >>> }
+            save_condition: True for save, False for depend on rt_config
+
+        Returns: 
+            None
 
-        :predicts: TODO
-        :rt_config: TODO
-        :condition: when the save condition is True, do save
-        :returns: TODO
         """
         if self.config.start_save_epoch == -1 or self.config.start_save_step == -1:
             self.config.start_save_step = rt_config.get('total_steps', 0) - 1
