@@ -1,3 +1,17 @@
+# Copyright 2021 cstsunfu. All rights reserved.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from dlk.utils.parser import config_parser_register
 import json
 import hjson
@@ -6,7 +20,7 @@ from dlk.data.processors import processor_config_register, processor_register
 
 
 class Processor(object):
-    """docstring for Processor"""
+    """Processor"""
     def __init__(self, config: Union[str, Dict]):
         super(Processor, self).__init__()
         self.config = config
@@ -17,9 +31,16 @@ class Processor(object):
             self.config = config[0]
         
     def fit(self, data: Dict[str, Any], stage='train'):
-        """TODO: Docstring for fit.
-        :returns: TODO
+        """Process the data and return the processed data
+
+        Args:
+            data: {"train": .., 'valid': ..}
+            stage: "train"/ 'predict', etc.
+
+        Returns: 
+            processed data
 
         """
+        
         processor = processor_register.get(self.config.get('_name'))(stage=stage, config=processor_config_register.get(self.config.get('_name'))(stage=stage, config=self.config))
-        processor.process(data)
+        return processor.process(data)
